#!/perl/bin/perl -w
use strict;
use v5.012;

use Data::Dumper;

use Encode;
use XML::Feed;
use HTML::TokeParser::Simple;
use Text::Wrap;

my %cfg = (
	'feedheader'   => "[0;37;0m[1;1H[2J\e[0;36m=================\e[1;37m[\e[1;32mNewscenter Update On\e[1;37m:  \e[1;36m__TIMESTAMP__\e[1;37m]\e[0;36m==================\e[0;37m\n",
	'feedsource'   => "__SOURCE1__\n\n",
	'feedheadline' => "__HEADLINE__\n",
	'feedteaser'   => "__TEASER__\n",
	'feedfooter'   => "\n",

	'tickheader'   => "[0;37;0m[1;1H[2J\e[0;36m=================\e[1;37m[\e[1;32mNewscenter Update On\e[1;37m:  \e[1;36m__TIMESTAMP__\e[1;37m]\e[0;36m==================\e[0;37m\n",
	'ticksource'   => "__SOURCE2__> ",
	'tickheadline' => "__HEADLINE__",
	'tickteaser'   => "\n",
	'tickfooter'   => "\n",

	'codepage'      => "cp437",
	'wordwrap'      => 76,
);

my %feedlist = (
  'Heise'        => 'http://www.heise.de/newsticker/heise-atom.xml',
  'Wired'        => 'http://feeds.wired.com/wired/index',
  'TMZ'          => 'http://www.tmz.com/rss.xml',
  'TechCrunch'   => 'http://feeds.feedburner.com/TechCrunch/',
  'Tagesschau'   => 'http://www.tagesschau.de/xml/rss2',
  'Stern'        => 'http://www.stern.de/feed/standard/alle-nachrichten/',
  'Sportschau'   => 'http://www.sportschau.de/sportschauindex100_type-rss.feed',
  'NFL'          => 'http://sports.espn.go.com/espn/rss/nfl/news',
  'Welt'         => 'http://www.welt.de/?service=Rss',
  'Nascar'       => 'http://sports.yahoo.com/nascar/rss.xml',
  'N24'          => 'http://feeds.n24.de/n24/homepage?format=xml',
  'Gulli'        => 'http://ticker.gulli.com/rss/',
  'Gamestar'     => 'http://rss.feedsportal.com/c/370/f/5299/index.rss',
  'Formel1'      => 'http://www.formula1.com/rss/news/latest.rss',
  'FLToday'      => 'http://rssfeeds.floridatoday.com/brevard/news',
  'ESPN'         => 'http://sports.espn.go.com/espn/rss/news',
  'CNN'          => 'http://rss.cnn.com/rss/cnn_world.rss',
  'Chip'         => 'http://rss.chip.de/c/573/f/7440/index.rss',
  'CCC'          => 'http://www.ccc.de/rss/updates.rdf',
  'BBC'          => 'http://feeds.bbci.co.uk/news/rss.xml',
  'ABC'          => 'http://feeds.abcnews.com/abcnews/topstories',
  '20min'        => 'http://www.20min.ch/rss/rss.tmpl?type=channel&get=1',
  'Wikinews'     => 'http://tools.wmflabs.org/dewikinews-rss/',
  'Post'         => 'http://feeds.washingtonpost.com/rss/rss_election-2012',
  'Postillion'   => 'http://feeds.feedburner.com/blogspot/rkEL?format=xml',
  'Titanic'      => 'http://www.titanic-magazin.de/ich.war.bei.der.waffen.rss',
  'Spiegel'      => 'http://www.spiegel.de/schlagzeilen/tops/index.rss',
  'Bild'         => 'http://feeds.feedburner.com/bild.xml',
  'Express'      => 'http://rss.feedsportal.com/c/32139/f/646634/index.rss',
  'FAZ'          => 'http://www.faz.net/rss/aktuell/',
  'WAZ'          => 'http://www.waz-online.de/rss/feed/waz_schlagzeilen',
  'SZ'           => 'http://suche.sueddeutsche.de/?output=rss',
  'BZ'           => 'http://bz-berlin.feedsportal.com/c/32933/fe.ed/www.bz-berlin.de/plainrss',
  'Focus'        => 'http://rss2.focus.de/c/32191/f/443312/index.rss',
  'NZZ'          => 'http://www.nzz.ch/feeds/recent.rss',
  'Tagesanzeiger'=> 'http://www.tagesanzeiger.ch/schweiz/rss.html',
  'Blick'        => 'http://www.blick.ch/news/rss',
  'Standard'     => 'http://derStandard.at/?page=rss&ressort=Seite1',
  'Kurier'       => 'http://kurier.at/newsfeed/nachrichten_rss.xml',
  'Wiener'       => 'http://www.wienerzeitung.at/_export/rss/channel/wien/index.rss',
  'Krone'        => 'http://www.krone.at/krone/S32/kmprog/rss.html',
  'PCWelt'       => 'http://pcwelt.feedsportal.com/c/269/f/534458/index.rss',
  'AndroidPIT'   => 'http://www.androidpit.de/feed/news.xml',
  'Macwelt'      => 'http://www.macwelt.de/rss/news-4394892.xml',
  'Tabtech'      => 'http://feeds.feedburner.com/Tabtechde?format=xml',
  'Onlinekosten' => 'http://onlinekosten.feedsportal.com/c/33775/f/604810/index.rss',
  'Fussball'     => 'http://rss.kicker.de/news/fussball',
  'Sport'        => 'http://www.sport.de/feed/579/telegramm.xml',
  'FTeu'         => 'http://www.ft.com/rss/world/europe',
  'FTus'         => 'http://www.ft.com/rss/world/americas',
  'FTuk'         => 'http://www.ft.com/rss/companies/uk',
  'Bzeitung'     => 'https://www.boersen-zeitung.de/xml_base/rss_bz.php?artsubm=unb',
  'ForbesMarket' => 'http://www.forbes.com/markets/index.xml',
  'ForbesAsia'   => 'http://www.forbes.com/asia_news/index.xml',
  'USAToday'     => 'http://rssfeeds.usatoday.com/usatoday-NewsTopStories',
);

# =========================================================================
sub fix_chars($) {
	my $x = $_[0];

	# missing CP437 characters
	$x =~ s/\x{a9}/(c)/g;
	$x =~ s/\x{2013}/-/g;
	$x =~ s/\x{2014}/-/g;
	$x =~ s/\x{2018}/'/g;
	$x =~ s/\x{2019}/'/g;
	$x =~ s/\x{201c}/"/g;
	$x =~ s/\x{201d}/"/g;
	$x =~ s/\x{201e}/"/g;
	$x =~ s/\x{2020}/+/g;		# died
	$x =~ s/\x{2026}/.../g;
	$x =~ s/\x{20ac}/EUR/g;
	$x =~ s/\x{2028}//g;		# LINE SEPARATOR

	return($x);
}

sub fix_feed($) {
	my $x = $_[0];

	$x =~ tr/ \t//s;	# no multiple spaces
	$x =~ tr/\n//s;		# no empty lines
	$x =~ s/[\r\n]//g;	# no newline/carriage return
	$x =~ s/^[\s]+//;	# no leading spaces
	$x =~ s/[\s]+$//;	# no trailing spaces

	$x =~ s/Read More$//;		# TechCrunch
	$x =~ s/\[Weiterlesen\]$//;	# Postillion
	$x =~ s/\(Dies ist ein Teaser - klick hier, um den kompletten Beitrag zu lesen\)$//; # AndroidPIT

	# Tagesschau fixes...
	$x =~ s/ \[(mehr|video|bilder|blog|livestream|live-ticker)\].*$//i;
	$x =~ s/\[(br|hr|mdr|ndr|rb|rbb|sr|swr|wdr|dw)\].*$/($1)/i;

	return($x);
}

sub strip_html($) {
	my $text = "";
	my $p = HTML::TokeParser::Simple->new(\$_[0]);
	while(my $t = $p->get_token) {
		next unless $t->is_text;
		$text .= $t->as_is;
	}

	# *sigh* WHY NOT SIMPLE ?!?
	$text =~ s/&nbsp;/ /g;			# space
	$text =~ s/&quot;/"/g;			# "
	$text =~ s/&auml;/\x{e4}/g;		# ae
	$text =~ s/&Auml;/\x{c4}/g;		# Ae
	$text =~ s/&ouml;/\x{f6}/g;		# oe
	$text =~ s/&Ouml;/\x{d6}/g;		# Oe
	$text =~ s/&uuml;/\x{fc}/g;		# ue
	$text =~ s/&Uuml;/\x{dc}/g;		# Ue
	$text =~ s/&szlig;/\x{df}/g;		# sz
	$text =~ s/&copy;/\x{a9}/g;		# (C)
	$text =~ s/&euro;/\x{20ac}/g;		# EUR
	$text =~ s/&hellip;/.../g;		# ...

	$text =~ s/&#(\d+);/chr($1)/eg;		# numbered html entities

	return($text);
}

$Text::Wrap::columns = $cfg{wordwrap};
my (@feeds, %templates);

# argument checks
if(($#ARGV < 1) || !($ARGV[0] =~ /(TICKER|FEED|FILEFEED)/i)) {
	say STDERR "Usage:";
	say STDERR "\t$0 <MODE> <NUM> <FEED...>";
	say STDERR "MODE: FEED | TICKER | FILEFEED";
	say STDERR "NUM:  number of news to show";
	say STDERR "Examples:";
	say STDERR "\t$0 FEED 4 Tagesschau";
	say STDERR "\t$0 TICKER 1 ABC BBC CNN";
	say STDERR "\t$0 FILEFEED 4 ABC BBC CNN Tagesschau";
	exit;
}

die("Error: Invalid arguments!\n") if($#ARGV < 2);
die("Error: Invalid number!\n") unless($ARGV[1] =~ /\d+/);

if($ARGV[0] =~ /FEED/i) {
	%templates = (
		'header'   => $cfg{feedheader},
		'source'   => $cfg{feedsource},
		'headline' => $cfg{feedheadline},
		'teaser'   => $cfg{feedteaser},
		'footer'   => $cfg{feedfooter},
	);
}

if($ARGV[0] =~ /FILEFEED/i) {
	%templates = (
		'header'   => $cfg{feedheader},
		'source'   => $cfg{feedsource},
		'headline' => $cfg{feedheadline},
		'teaser'   => $cfg{feedteaser},
		'footer'   => $cfg{feedfooter},
		'fileout'  => 1,
	);
}

if($ARGV[0] =~ /TICKER/i) {
	%templates = (
		'header'   => $cfg{tickheader},
		'source'   => $cfg{ticksource},
		'headline' => $cfg{tickheadline},
		'teaser'   => $cfg{tickteaser},
		'footer'   => $cfg{tickfooter},
	);
}

@feeds = @ARGV[2 .. $#ARGV];
foreach(@feeds) {
	die("Error: Invalid feed $_!\n") if(!$feedlist{$_});
}

# handle all feeds
my $output = "";
foreach my $ff (@feeds) {
	my $tmp;
	my $feed = XML::Feed->parse(URI->new($feedlist{$ff}));
	warn("Warning ($ff): " . XML::Feed->errstr . "\n") if(!$feed);
	next if(!$feed);

	# print feed name
	my $feedtitle = $feed->title;
	$tmp = $templates{source};
	$tmp =~ s/__SOURCE1__/$feedtitle/;
	$tmp =~ s/__SOURCE2__/$ff/;
	$tmp = encode($cfg{codepage}, $tmp);
	$output .= $tmp;

	# print entries
	my $num = 0;
	my $tries = 2 * $ARGV[1] + 10;
	my @entries = $feed->entries;
	for(my $index = 0; $num < $ARGV[1] && $tries; $index++) {
		$tries--;

		# fetch and handle title
		next unless($entries[$index]);
		my $title = $entries[$index]->title;
		$title =~ s/\n//g if($title);	# remove newlines (N24)

		# fetch and handle body
		my $body = $entries[$index]->content->body;
		my $type = $entries[$index]->content->type;

		# skip if undefined title or body
		next unless defined $title;
		next unless defined $body;

		$body = decode('UTF-8', $body) unless(Encode::is_utf8($body));
		if($type eq 'text/html') {
			$body = strip_html($body);
		}

		# apply fixes
		$title = fix_chars($title);
		$body  = fix_chars($body);
		$body  = fix_feed($body);

		# limit ticker line length
		if($ARGV[0] =~ /TICKER/i) {
			my $len = 76 - length($ff);
			$title = substr($title, 0, $len);
		}

		# wrap long lines
		$title = wrap('', '', $title) if(length($title) > 79);
		$body  = wrap('', '', $body);

		# skip if body not empty
		next if($body =~ /^\s?$/);

		# encode to single-byte encoding
		$title = encode($cfg{codepage}, $title);
		$body  = encode($cfg{codepage}, $body);

		# print headline
		$tmp = $templates{headline};
		$tmp =~ s/__HEADLINE__/$title/;
		$output .= $tmp;

		# print teaser
		$tmp = $templates{teaser};
		$tmp =~ s/__TEASER__/$body/;
		$output .= $tmp;

		$num++;
	}

	# FILEFEED: write feed to textfile and timestamped log to stdout
	if($templates{fileout}) {
		my ($sec, $min, $hour, $mday, $mon, $yr) = localtime(time);
		my $timestamp = sprintf("%04d-%02d-%02d %02d:%02d:%02d",
			$yr+1900, $mon+1, $mday, $hour, $min, $sec);
		my $header = $templates{header};
		   $header =~ s/__TIMESTAMP__/$timestamp/;
		my $footer = $templates{footer};
		   $footer =~ s/__TIMESTAMP__/$timestamp/;
		say "$ff - $timestamp";

		# insert timestamp in output
		$timestamp =~ s/:\d{2}$//;
		$output    =~ s/\n/ ($timestamp)\n/m;

		open(OUTFILE,">","$ff.txt") or die("Can't open $ff.txt!");
		print OUTFILE $header . $output . $footer;
		$output = "";
		close(OUTFILE);
	}
}

if(!$templates{fileout}) {
	# calculate timestamp
	my ($sec, $min, $hour, $mday, $mon, $yr) = localtime(time);
	my $timestamp = sprintf("%02d.%02d.%04d - %02d:%02d:%02d",
		$mday, $mon+1, $yr+1900, $hour, $min, $sec);
	my $header = $templates{header};
	   $header =~ s/__TIMESTAMP__/$timestamp/;
	my $footer = $templates{footer};
	   $footer =~ s/__TIMESTAMP__/$timestamp/;

	# finally print output
	print $header . $output . $footer;
}

